@@ -1,15 +1,46 @@
+# This file is part of khmer, https://github.com/dib-lab/khmer/, and is
+# Copyright (C) 2010-2015, Michigan State University.
+# Copyright (C) 2015-2016, The Regents of the University of California.
+#
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are
+# met:
+#
+#     * Redistributions of source code must retain the above copyright
+#       notice, this list of conditions and the following disclaimer.
+#
+#     * Redistributions in binary form must reproduce the above
+#       copyright notice, this list of conditions and the following
+#       disclaimer in the documentation and/or other materials provided
+#       with the distribution.
+#
+#     * Neither the name of the Michigan State University nor the names
+#       of its contributors may be used to endorse or promote products
+#       derived from this software without specific prior written
+#       permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+# HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #
-# This file is part of khmer, http://github.com/ged-lab/khmer/, and is
-# Copyright (C) Michigan State University, 2009-2013. It is licensed under
-# the three-clause BSD license; see doc/LICENSE.txt.
 # Contact: khmer-project@idyll.org
-#
-# pylint: disable=missing-docstring
+# pylint: disable=missing-docstring,invalid-name,no-member,no-self-use
+# pylint: disable=protected-access
+from __future__ import print_function, absolute_import
+
 import khmer
 import screed
 
-import khmer_tst_utils as utils
-from nose.plugins.attrib import attr
+import os
+from . import khmer_tst_utils as utils
 
 
 def teardown():
@@ -19,14 +50,16 @@ def teardown():
 class Test_RandomData(object):
 
     def test_3_merge_013(self):
-        ht = khmer.new_hashbits(20, 4 ** 4 + 1)
+        ht = khmer.Nodegraph(20, 4 ** 4 + 1, 2)
 
         filename = utils.get_test_data('test-graph2.fa')
 
-        (total_reads, total_kmers) = ht.consume_fasta_and_tag(filename)
+        (total_reads, _) = ht.consume_seqfile_and_tag(filename)
         assert total_reads == 3, total_reads
 
-        (a, b, c) = ht.divide_tags_into_subsets(1)
+        divvy = ht.divide_tags_into_subsets(1)
+        assert len(divvy) is 3
+        (a, b, _) = divvy
 
         x = ht.do_subset_partition(a, a)
         ht.merge_subset(x)
@@ -39,13 +72,15 @@ class Test_RandomData(object):
         assert n_partitions == 1, n_partitions        # combined.
 
     def test_3_merge_023(self):
-        ht = khmer.new_hashbits(20, 4 ** 4 + 1)
+        ht = khmer.Nodegraph(20, 4 ** 4 + 1, 2)
         filename = utils.get_test_data('test-graph2.fa')
 
-        (total_reads, total_kmers) = ht.consume_fasta_and_tag(filename)
+        (total_reads, _) = ht.consume_seqfile_and_tag(filename)
         assert total_reads == 3, total_reads
 
-        (a, b, c) = ht.divide_tags_into_subsets(1)
+        divvy = ht.divide_tags_into_subsets(1)
+        assert len(divvy) is 3
+        (a, b, c) = divvy
 
         x = ht.do_subset_partition(b, c)
         ht.merge_subset(x)
@@ -58,13 +93,14 @@ class Test_RandomData(object):
         assert n_partitions == 1, n_partitions        # combined.
 
     def test_5_merge_046(self):
-        ht = khmer.new_hashbits(20, 4 ** 4 + 1)
+        ht = khmer.Nodegraph(20, 4 ** 4 + 1, 2)
         filename = utils.get_test_data('test-graph5.fa')
 
-        (total_reads, total_kmers) = ht.consume_fasta_and_tag(filename)
+        (total_reads, _) = ht.consume_seqfile_and_tag(filename)
         assert total_reads == 6, total_reads
 
         divvy = ht.divide_tags_into_subsets(1)
+        divvy = list(divvy)
 
         x = ht.do_subset_partition(divvy[0], divvy[4])
         ht.merge_subset(x)
@@ -77,14 +113,15 @@ class Test_RandomData(object):
         assert n_partitions == 1, n_partitions        # combined.
 
     def test_random_20_a_succ(self):
-        ht = khmer.new_hashbits(20, 4 ** 7 + 1)
+        ht = khmer.Nodegraph(20, 4 ** 7 + 1, 2)
         filename = utils.get_test_data('random-20-a.fa')
         outfile = utils.get_temp_filename('out')
 
-        total_reads, _ = ht.consume_fasta_and_tag(filename)
+        total_reads, _ = ht.consume_seqfile_and_tag(filename)
 
-        subset_size = total_reads / 2 + total_reads % 2
+        subset_size = total_reads // 2 + total_reads % 2
         divvy = ht.divide_tags_into_subsets(subset_size)
+        divvy = list(divvy)
         assert len(divvy) == 4
 
         x = ht.do_subset_partition(divvy[0], divvy[2])
@@ -96,14 +133,15 @@ class Test_RandomData(object):
         assert n_partitions == 1, n_partitions
 
     def test_random_20_a_succ_II(self):
-        ht = khmer.new_hashbits(20, 4 ** 7 + 1)
+        ht = khmer.Nodegraph(20, 4 ** 7 + 1, 2)
         filename = utils.get_test_data('random-20-a.fa')
         outfile = utils.get_temp_filename('out')
 
-        total_reads, _ = ht.consume_fasta_and_tag(filename)
+        total_reads, _ = ht.consume_seqfile_and_tag(filename)
 
-        subset_size = total_reads / 2 + total_reads % 2
+        subset_size = total_reads // 2 + total_reads % 2
         divvy = ht.divide_tags_into_subsets(subset_size)
+        divvy = list(divvy)
         assert len(divvy) == 4
 
         x = ht.do_subset_partition(divvy[0], divvy[2])
@@ -115,14 +153,15 @@ class Test_RandomData(object):
         assert n_partitions == 1, n_partitions
 
     def test_random_20_a_succ_III(self):
-        ht = khmer.new_hashbits(20, 4 ** 7 + 1)
+        ht = khmer.Nodegraph(20, 4 ** 7 + 1, 2)
         filename = utils.get_test_data('random-20-a.fa')
         outfile = utils.get_temp_filename('out')
 
-        total_reads, _ = ht.consume_fasta_and_tag(filename)
+        total_reads, _ = ht.consume_seqfile_and_tag(filename)
 
-        subset_size = total_reads / 2 + total_reads % 2
+        subset_size = total_reads // 2 + total_reads % 2
         divvy = ht.divide_tags_into_subsets(subset_size)
+        divvy = list(divvy)
         assert len(divvy) == 4, len(divvy)
 
         x = ht.do_subset_partition(divvy[0], divvy[2])
@@ -138,14 +177,15 @@ class Test_RandomData(object):
         assert n_partitions == 1, n_partitions
 
     def test_random_20_a_succ_IV(self):
-        ht = khmer.new_hashbits(20, 4 ** 7 + 1)
+        ht = khmer.Nodegraph(20, 4 ** 7 + 1, 2)
         filename = utils.get_test_data('random-20-a.fa')
         outfile = utils.get_temp_filename('out')
 
-        total_reads, _ = ht.consume_fasta_and_tag(filename)
+        ht.consume_seqfile_and_tag(filename)
         subsets = []
 
         divvy = ht.divide_tags_into_subsets(1)
+        divvy = list(divvy)
         divvy.append(0)
         for i in range(len(divvy) - 1):
             x = ht.do_subset_partition(divvy[i], divvy[i + 1])
@@ -158,25 +198,26 @@ class Test_RandomData(object):
         assert n_partitions == 1, n_partitions
 
     def test_random_20_a_succ_IV_save(self):
-        ht = khmer.new_hashbits(20, 4 ** 7 + 1)
+        ht = khmer.Nodegraph(20, 4 ** 7 + 1, 2)
         filename = utils.get_test_data('random-20-a.fa')
 
         savefile_ht = utils.get_temp_filename('ht')
         savefile_tags = utils.get_temp_filename('tags')
         outfile = filename + utils.get_temp_filename('out')
 
-        total_reads, _ = ht.consume_fasta_and_tag(filename)
+        ht.consume_seqfile_and_tag(filename)
 
         ht.save(savefile_ht)
         ht.save_tagset(savefile_tags)
 
         del ht
-        ht = khmer.new_hashbits(20, 4 ** 7 + 1)
+        ht = khmer.Nodegraph(20, 4 ** 7 + 1, 2)
 
         ht.load(savefile_ht)
         ht.load_tagset(savefile_tags)
 
         divvy = ht.divide_tags_into_subsets(1)
+        divvy = list(divvy)
         divvy.append(0)
 
         subsets = []
@@ -194,15 +235,15 @@ class Test_RandomData(object):
 class Test_SaveLoadPmap(object):
 
     def test_save_load_merge(self):
-        ht = khmer.new_hashbits(20, 4 ** 4 + 1)
+        ht = khmer.Nodegraph(20, 4 ** 4 + 1, 2)
         filename = utils.get_test_data('test-graph2.fa')
 
-        (total_reads, total_kmers) = ht.consume_fasta_and_tag(filename)
+        (total_reads, _) = ht.consume_seqfile_and_tag(filename)
         assert total_reads == 3, total_reads
 
         divvy = ht.divide_tags_into_subsets(1)
-        print divvy
-        (a, b, c) = divvy
+        assert len(divvy) == 3
+        (a, b, _) = divvy
 
         outfile1 = utils.get_temp_filename('x.pmap')
         outfile2 = utils.get_temp_filename('y.pmap')
@@ -225,14 +266,52 @@ class Test_SaveLoadPmap(object):
         n_partitions = ht.output_partitions(filename, outfile)
         assert n_partitions == 1, n_partitions        # combined.
 
+    def test_save_load_merge_truncate(self):
+        ht = khmer.Nodegraph(20, 4 ** 4 + 1, 2)
+        filename = utils.get_test_data('test-graph2.fa')
+
+        (total_reads, _) = ht.consume_seqfile_and_tag(filename)
+        assert total_reads == 3, total_reads
+
+        divvy = ht.divide_tags_into_subsets(1)
+        print(divvy)
+        assert len(divvy) is 3
+        (a, b, _) = divvy
+
+        outfile1 = utils.get_temp_filename('x.pmap')
+        outfile2 = utils.get_temp_filename('y.pmap')
+
+        x = ht.do_subset_partition(a, b)
+        ht.save_subset_partitionmap(x, outfile1)
+        del x
+
+        y = ht.do_subset_partition(b, 0)
+        ht.save_subset_partitionmap(y, outfile2)
+        del y
+
+        outfile3 = utils.get_temp_filename('z.pmap')
+        data = open(outfile1, 'rb').read()
+
+        for i in range(len(data)):
+            fp = open(outfile3, 'wb')
+            fp.write(data[:i])
+            fp.close()
+
+            try:
+                a = ht.load_subset_partitionmap(outfile3)
+                assert 0, "this should not pass"
+            except OSError as err:
+                print(str(err), i)
+
     def test_save_load_merge_2(self):
-        ht = khmer.new_hashbits(20, 4 ** 8 + 1)
+        ht = khmer.Nodegraph(20, 4 ** 8 + 1, 2)
         filename = utils.get_test_data('random-20-a.fa')
 
-        (total_reads, total_kmers) = ht.consume_fasta_and_tag(filename)
+        (total_reads, _) = ht.consume_seqfile_and_tag(filename)
 
-        subset_size = total_reads / 2 + total_reads % 2
+        subset_size = total_reads // 2 + total_reads % 2
         divvy = ht.divide_tags_into_subsets(subset_size)
+        divvy = list(divvy)
 
         outfile1 = utils.get_temp_filename('x.pmap')
         outfile2 = utils.get_temp_filename('y.pmap')
@@ -245,6 +324,8 @@ class Test_SaveLoadPmap(object):
         ht.save_subset_partitionmap(y, outfile2)
         del y
 
+        assert os.path.exists(outfile1)
+        assert os.path.exists(outfile2)
         a = ht.load_subset_partitionmap(outfile1)
         b = ht.load_subset_partitionmap(outfile2)
 
@@ -256,23 +337,23 @@ class Test_SaveLoadPmap(object):
         assert n_partitions == 1, n_partitions        # combined.
 
     def test_save_load_merge_nexist(self):
-        ht = khmer.new_hashbits(20, 1)
+        ht = khmer._Nodegraph(20, [1])
         try:
-            a = ht.load_subset_partitionmap('this does not exist')
+            ht.load_subset_partitionmap('this does not exist')
             assert 0, "this should not succeed"
-        except IOError as e:
-            print str(e)
+        except OSError as e:
+            print(str(e))
 
     def test_save_merge_from_disk(self):
-        ht = khmer.new_hashbits(20, 4 ** 4 + 1)
+        ht = khmer.Nodegraph(20, 4 ** 4 + 1, 2)
         filename = utils.get_test_data('test-graph2.fa')
 
-        (total_reads, total_kmers) = ht.consume_fasta_and_tag(filename)
+        (total_reads, _) = ht.consume_seqfile_and_tag(filename)
         assert total_reads == 3, total_reads
 
         divvy = ht.divide_tags_into_subsets(1)
-        print divvy
-        (a, b, c) = divvy
+        print(divvy)
+        (a, b, _) = divvy
 
         outfile1 = utils.get_temp_filename('x.pmap')
         outfile2 = utils.get_temp_filename('y.pmap')
@@ -293,13 +374,14 @@ class Test_SaveLoadPmap(object):
         assert n_partitions == 1, n_partitions        # combined.
 
     def test_save_merge_from_disk_2(self):
-        ht = khmer.new_hashbits(20, 4 ** 7 + 1)
+        ht = khmer.Nodegraph(20, 4 ** 7 + 1, 2)
         filename = utils.get_test_data('random-20-a.fa')
 
-        (total_reads, total_kmers) = ht.consume_fasta_and_tag(filename)
+        (total_reads, _) = ht.consume_seqfile_and_tag(filename)
 
-        subset_size = total_reads / 2 + total_reads % 2
+        subset_size = total_reads // 2 + total_reads % 2
         divvy = ht.divide_tags_into_subsets(subset_size)
+        divvy = list(divvy)
 
         outfile1 = utils.get_temp_filename('x.pmap')
         outfile2 = utils.get_temp_filename('y.pmap')
@@ -312,6 +394,8 @@ class Test_SaveLoadPmap(object):
         ht.save_subset_partitionmap(y, outfile2)
         del y
 
+        assert os.path.exists(outfile1)
+        assert os.path.exists(outfile2)
         ht.merge_subset_from_disk(outfile1)
         ht.merge_subset_from_disk(outfile2)
 
@@ -320,16 +404,12 @@ class Test_SaveLoadPmap(object):
         assert n_partitions == 1, n_partitions        # combined.
 
     def test_save_merge_from_disk_file_not_exist(self):
-        ht = khmer.new_hashbits(20, 4 ** 4 + 1)
+        ht = khmer.Nodegraph(20, 4 ** 4 + 1, 2)
         filename = utils.get_test_data('test-graph2.fa')
 
-        (total_reads, total_kmers) = ht.consume_fasta_and_tag(filename)
+        (total_reads, _) = ht.consume_seqfile_and_tag(filename)
         assert total_reads == 3, total_reads
 
-        divvy = ht.divide_tags_into_subsets(1)
-        print divvy
-        (a, b, c) = divvy
-
         outfile1 = utils.get_temp_filename('x.pmap')
 
         # fail to create file... => failure expected
@@ -337,57 +417,128 @@ class Test_SaveLoadPmap(object):
         try:
             ht.merge_subset_from_disk(outfile1)
             assert 0, "this should fail"
-        except IOError as e:
-            print str(e)
+        except OSError as e:
+            print(str(e))
 
     def test_merge_from_disk_file_bad_type(self):
-        ht = khmer.new_hashbits(20, 4 ** 4 + 1)
+        ht = khmer.Nodegraph(20, 4 ** 4 + 1, 2)
         infile = utils.get_test_data('goodversion-k12.ht')
 
         try:
             ht.merge_subset_from_disk(infile)
             assert 0, "this should fail"
-        except IOError as e:
-            print str(e)
+        except OSError as e:
+            print(str(e))
 
     def test_merge_from_disk_file_version(self):
-        ht = khmer.new_hashbits(20, 4 ** 4 + 1)
+        ht = khmer.Nodegraph(20, 4 ** 4 + 1, 2)
         infile = utils.get_test_data('badversion-k12.ht')
 
         try:
             ht.merge_subset_from_disk(infile)
             assert 0, "this should fail"
-        except IOError as e:
-            print str(e)
+        except OSError as e:
+            print(str(e))
 
     def test_save_merge_from_disk_ksize(self):
-        ht = khmer.new_hashbits(20, 4 ** 4 + 1)
+        ht = khmer.Nodegraph(20, 4 ** 4 + 1, 2)
         filename = utils.get_test_data('test-graph2.fa')
 
-        (total_reads, total_kmers) = ht.consume_fasta_and_tag(filename)
+        (total_reads, _) = ht.consume_seqfile_and_tag(filename)
         assert total_reads == 3, total_reads
 
         divvy = ht.divide_tags_into_subsets(1)
-        print divvy
-        (a, b, c) = divvy
+        print(divvy)
+        (a, b, _) = divvy
 
         outfile1 = utils.get_temp_filename('x.pmap')
         x = ht.do_subset_partition(a, b)
         ht.save_subset_partitionmap(x, outfile1)
         del x
 
-        ht = khmer.new_hashbits(19, 1, 1)
+        ht = khmer._Nodegraph(19, [1])
         try:
             ht.merge_subset_from_disk(outfile1)
             assert 0, "this should fail"
-        except IOError as e:
-            print str(e)
+        except OSError as e:
+            print(str(e))
+
+
+def test_save_load_merge_on_graph():
+    ht = khmer.Nodegraph(20, 4 ** 4 + 1, 2)
+    filename = utils.get_test_data('test-graph2.fa')
+
+    (total_reads, _) = ht.consume_seqfile_and_tag(filename)
+    assert total_reads == 3, total_reads
+
+    divvy = ht.divide_tags_into_subsets(1)
+    print(divvy)
+    assert len(divvy) is 3
+    (a, b, _) = divvy
+
+    outfile1 = utils.get_temp_filename('x.pmap')
+    outfile2 = utils.get_temp_filename('y.pmap')
+
+    x = ht.do_subset_partition(a, b)
+    ht.save_subset_partitionmap(x, outfile1)
+    del x
+
+    y = ht.do_subset_partition(b, 0)
+    ht.save_subset_partitionmap(y, outfile2)
+    del y
+
+    a = ht.load_partitionmap(outfile1)  # <-- this is different
+    b = ht.load_subset_partitionmap(outfile2)
+
+    ht.merge_subset(b)
+
+    outfile = utils.get_temp_filename('out.part')
+    n_partitions = ht.output_partitions(filename, outfile)
+    assert n_partitions == 1, n_partitions        # combined.
+
+
+def test_save_load_on_graph_truncate():
+    ht = khmer.Nodegraph(20, 4 ** 4 + 1, 2)
+    filename = utils.get_test_data('test-graph2.fa')
+
+    (total_reads, _) = ht.consume_seqfile_and_tag(filename)
+    assert total_reads == 3, total_reads
+
+    divvy = ht.divide_tags_into_subsets(1)
+    print(divvy)
+    assert len(divvy) is 3
+    (a, b, _) = divvy
+
+    outfile1 = utils.get_temp_filename('x.pmap')
+    outfile2 = utils.get_temp_filename('y.pmap')
+
+    x = ht.do_subset_partition(a, b)
+    ht.save_subset_partitionmap(x, outfile1)
+    del x
+
+    y = ht.do_subset_partition(b, 0)
+    ht.save_subset_partitionmap(y, outfile2)
+    del y
+
+    outfile3 = utils.get_temp_filename('z.pmap')
+    data = open(outfile1, 'rb').read()
+
+    for i in range(len(data)):
+        fp = open(outfile3, 'wb')
+        fp.write(data[:i])
+        fp.close()
+
+        try:
+            a = ht.load_partitionmap(outfile3)
+            assert 0, "this should not pass"
+        except OSError as err:
+            print(str(err), i)
 
 
 def test_output_partitions():
     filename = utils.get_test_data('test-output-partitions.fa')
 
-    ht = khmer.new_hashbits(10, 1, 1)
+    ht = khmer._Nodegraph(10, [1])
     ht.set_partition_id('TTAGGACTGC', 2)
     ht.set_partition_id('TGCGTTTCAA', 3)
     ht.set_partition_id('ATACTGTAAA', 4)
@@ -412,8 +563,8 @@ test_output_partitions.runme = True
 def test_tiny_real_partitions():
     filename = utils.get_test_data('real-partition-tiny.fa')
 
-    ht = khmer.new_hashbits(32, 8e1, 4)
-    ht.consume_fasta_and_tag(filename)
+    ht = khmer.Nodegraph(32, 8e2, 4)
+    ht.consume_seqfile_and_tag(filename)
 
     subset = ht.do_subset_partition(0, 0)
     ht.merge_subset(subset)
@@ -439,8 +590,8 @@ def test_tiny_real_partitions():
 def test_small_real_partitions():
     filename = utils.get_test_data('real-partition-small.fa')
 
-    ht = khmer.new_hashbits(32, 2e2, 4)
-    ht.consume_fasta_and_tag(filename)
+    ht = khmer.Nodegraph(32, 2e3, 4)
+    ht.consume_seqfile_and_tag(filename)
 
     subset = ht.do_subset_partition(0, 0)
     ht.merge_subset(subset)
@@ -461,7 +612,7 @@ def test_small_real_partitions():
 
 test_small_real_partitions.runme = True
 
-a = """\
+first = """\
 CAGACTTGGAAGCTGAGAGTCCGACGTCACTGCCTCAACTCGCGCAAATGTTCCCGCCAA\
 ATTGTATCCTAGGGATCTTCCATAAGCTTATATACGGGGGTTTCCAAGGCCCTGATGCCA\
 GTGCCTAATCTTTTGGAGTCCTCTCAGGGCCACTAGATGCCATGCTACGCGTCCCAGGTT\
@@ -469,7 +620,7 @@ GGCCTGAGGGTCTACACGGAGTGGGAAGCATGGGTACCTTAGCGAACATTCATACTGGCC\
 TGTTTATGCTTATCAGACTTCAGCTTCGCTTAGCGCGTCACCGTTTGTAACTTGTTATCT\
 """
 
-b = """\
+second = """\
 TGTTTATGCTTATCAGACTTCAGCTTCGCTTAGCGCGTCACCGTTTGTAACTTGTTATCT\
 GACTGTAGACTTGAACCTCGATGGAATGCAGGTCCCATTCTCTGGCCTGACTCATGGAAC\
 CGAGGCCAAAAAAGCATGGCACGAAGACGCTATGCGAGGGTGCTCGCCCATGTCGTCGCC\
@@ -479,14 +630,12 @@ CCTCGGGCCTTTCCGTTCCGTTGCCGCCCAAGCTCTCTAGCATCGAATCGGTCAAGCGGT\
 
 
 def test_partition_on_abundance_1():
-    print(a,)
-    print(b,)
-    kh = khmer.new_counting_hash(20, 1e3, 4)
-    for i in range(10):
-        print kh.consume_and_tag(a)
+    kh = khmer.Countgraph(20, 1e3, 4)
+    for _ in range(10):
+        print(kh.consume_and_tag(first))
 
-    for i in range(10):
-        print kh.consume_and_tag(b)
+    for _ in range(10):
+        print(kh.consume_and_tag(second))
 
     # all paths in 'a' and 'b'
     p = kh.do_subset_partition_with_abundance(10, 50)
@@ -495,12 +644,12 @@ def test_partition_on_abundance_1():
 
 
 def test_partition_on_abundance_2():
-    kh = khmer.new_counting_hash(20, 1e3, 4)
-    for i in range(10):
-        print kh.consume_and_tag(a)
+    kh = khmer.Countgraph(20, 1e3, 4)
+    for _ in range(10):
+        print(kh.consume_and_tag(first))
 
-    for i in range(5):
-        print kh.consume_and_tag(b)
+    for _ in range(5):
+        print(kh.consume_and_tag(second))
 
     # all paths in 'a'
     p = kh.do_subset_partition_with_abundance(10, 50)
@@ -509,12 +658,12 @@ def test_partition_on_abundance_2():
 
 
 def test_partition_on_abundance_3():
-    kh = khmer.new_counting_hash(20, 1e4, 4)
-    for i in range(10):
-        print kh.consume_and_tag(a)
+    kh = khmer.Countgraph(20, 1e4, 4)
+    for _ in range(10):
+        print(kh.consume_and_tag(first))
 
-    for i in range(5):
-        print kh.consume_and_tag(b)
+    for _ in range(5):
+        print(kh.consume_and_tag(second))
 
     # this will get paths only in 'a'
     p = kh.do_subset_partition_with_abundance(10, 50)
@@ -523,38 +672,17 @@ def test_partition_on_abundance_3():
     p = kh.do_subset_partition_with_abundance(5, 10)
 
     x = p.count_partitions()
-    print x
+    print(x)
     assert x == (2, 2)                  # two partitions, two ignored tags
 
 
-def test_partition_overlap_1():
-    kh = khmer.new_counting_hash(20, 1e3, 4)
-    for i in range(10):
-        kh.consume_and_tag(a)
-
-    for i in range(10):
-        kh.consume_and_tag(b)
-
-    # this will get paths only in 'a'
-    p1 = kh.do_subset_partition_with_abundance(10, 50)
-
-    # this will get paths only in 'a', again -- should be the same!
-    p2 = kh.do_subset_partition_with_abundance(10, 50)
-
-    # p1.report_on_partitions()
-    # p2.report_on_partitions()
-
-    x = p1.compare_partitions(3, p2, 3)
-    assert x == (0, 0, 14), x
-
-
 def test_partition_overlap_2():
-    kh = khmer.new_counting_hash(20, 1e4, 4)
-    for i in range(10):
-        kh.consume_and_tag(a)
+    kh = khmer.Countgraph(20, 1e4, 4)
+    for _ in range(10):
+        kh.consume_and_tag(first)
 
-    for i in range(5):
-        kh.consume_and_tag(b)
+    for _ in range(5):
+        kh.consume_and_tag(second)
 
     # this will get paths only in 'a'
     p1 = kh.do_subset_partition_with_abundance(10, 50)
@@ -565,17 +693,11 @@ def test_partition_overlap_2():
     # p1.report_on_partitions()
     # p2.report_on_partitions()
 
-    x = p1.compare_partitions(3, p2, 3)
-    assert x == (8, 6, 0), x
-
-    x = p1.compare_partitions(3, p2, 5)
-    assert x == (2, 0, 6), x
-
     x = p1.partition_sizes()
     assert x == ([(3, 8)], 0), x
 
     x = p2.partition_sizes()
-    assert x == ([(3, 6), (5, 6)], 2), x
+    assert x == ([(3, 6), (5, 6)], 0), x
 
     x = p1.partition_average_coverages(kh)
     assert x == [(3, 11)]
